#include <algorithm>
#include <vector>
#include <sstream>
#include <string>
#include <iterator>

#include <iostream>
#include <fstream>
#include <ctime>            // std::time

#include <boost/random/normal_distribution.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/random/linear_congruential.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/triangular.hpp>
using namespace std;
using namespace boost::numeric::ublas;

typedef boost::minstd_rand base_generator_type;
// generating normally distributed data
struct ca {
	base_generator_type generator;
	boost::normal_distribution<double> norm_dist;
	boost::variate_generator<base_generator_type&, boost::normal_distribution<double> > norm;
	ca(double mean,double variance) : generator(24u),norm_dist(mean,variance),norm(generator,norm_dist) {}//24u, seed
	double operator()() {return norm();}
}RandomNumber(0,1) ;

//the function "bind" realize the bind for two matrix. equal to the function M=[A,B] or [A;B] in MATLAB 
matrix<double> bind (matrix<double> m1, matrix<double> m2, int idx){ // idx=1: colomn bind; idx=2: row bind
	matrix<double> m;
	if (idx==1){
		// for column bind, the # of rows of two matrices must be the same
		matrix<double> n(m1.size1(),m1.size2()+m2.size2());
		subrange(n,0,m1.size1(),0,m1.size2())=m1;
		subrange(n,0,m1.size1(),m1.size2(),m1.size2()+m2.size2())=m2;
		m=n;

	} else{
		// for row bind, the # of coloums of two matrices must be the same
		matrix<double> n(m1.size1()+m2.size1(),m1.size2());
		subrange(n,0,m1.size1(),0,m1.size2())=m1;
		subrange(n,m1.size1(),m2.size1()+m1.size1(),0,m1.size2())=m2;
		m=n;		
	}
	return m;
}

// convert a ublas vector to matrix form. The library can not convert them by itself
matrix<double> Convert_vector(boost::numeric::ublas::vector<double> v, int idx){ // idx==1: convert to 1*r; idx==2, convert to r*1
	matrix<double> n;
	if(idx==1){
	    matrix<double> m(1,v.size());
		row(m,0)=v;
		n=m;
	}else{
		matrix<double> m(1,v.size());
		column(m,0)=v;
		n=m;
	}
	return n;
}

// template <class T>
// std::vector<T> StringToVec( const std::string& Str );
// Tokenize a passed line/string into a vector
// e.g:
// std::string str = "1 2 3 4";
// std::vector<int> vec = StringToVector<int>(str);

template <class T>
std::vector<T> StringToVec( const string &str )
{
	std::vector<T> MyVec;
	istringstream iss (str, istringstream::in);
	copy(istream_iterator<T>(iss), istream_iterator<T>(), back_inserter(MyVec)); //check http://www.sgi.com/tech/stl/istream_iterator.html
	return MyVec;
}

// The code below converts a std::vector to a ublas::vector, so that we can read a file and write it in a matrix.
boost::numeric::ublas::vector<double> Convert_vector_to_matrix(std::vector<double> r){
	boost::numeric::ublas::vector<double> v(r.size());
	std::vector<double>::iterator Iter;
	int i=0;

	for(Iter = r.begin(); Iter !=r.end(); Iter++ ) {
		v(i)=*Iter;
		i++;
	}
	r.clear();
	return v;

}

// the following function helps to get the mean of each row or col
// the struct is used for the iterator to add things up

struct adder : public unary_function<double, void>{
	adder() : sum(0) {}
	double sum;
	void operator()(double x) { sum += x; }
};

boost::numeric::ublas::vector<double> Get_sum(matrix<double> m, int idx){//idx==1: sum for each row; idx==2; sum for each col
	boost::numeric::ublas::vector<double> sum_m;
	if(idx==1){//sum each row
		 boost::numeric::ublas::vector<double> v(m.size1());
		 for (unsigned i = 0; i < m.size1 (); i++) {
               matrix_row<matrix<double> > mr (m, i);
			   adder result_m=for_each(mr.begin(),mr.end(),adder());
			   v(i)=result_m.sum;
		 }
		 sum_m=v;
	}else{ //sum each col
		 boost::numeric::ublas::vector<double> v(m.size2());
		 for (unsigned i = 0; i < m.size2 (); i++) {
               matrix_column<matrix<double> > mr (m, i);
			    adder result_m=for_each(mr.begin(),mr.end(),adder());
				v(i)=result_m.sum;
		 }
		 sum_m=v;
	}
	return sum_m;
}

// mean is another form of sum. easy to handle
boost::numeric::ublas::vector<double> Get_mean(matrix<double> m, int idx){//idx==1: sum for each row; idx==2; sum for each col
	boost::numeric::ublas::vector<double> mean_m;
	int w=((idx==1)*m.size2()+(idx==2)*m.size1());
	mean_m=Get_sum(m,idx)/w;
	return mean_m;
}


// simple function sqrt(matrix<double> m). for each element, get sqrt
matrix<double> sqrt_m(matrix<double> m){
   matrix<double> r(m.size1(),m.size2());
	
	for (int i=0; i<m.size1(); i++)
		for (int j=0; j<m.size2(); j++)
			r(i,j)=sqrt(m(i,j));
	return r;
}
//////////////////////////////////////////////////needs to be fixed, similar to adder

//compute the covariate matrix of X
matrix<double> Get_cc(matrix<double> m){
	int N=m.size1();
	int dim1=m.size2();
	matrix<double> tmp1(m.size1(),1);
	//  ones(N,1)
	for (int i=0; i<m.size1(); i++)
		tmp1(i,0)=1;
	matrix<double> tmp2=prod(tmp1,Convert_vector(Get_mean(m,2),1));
	m=m-tmp2;    //center the data;
	matrix<double> tmp_norm= element_prod(m, m);
	tmp_norm=sqrt_m(Convert_vector(Get_sum(tmp_norm,2),1));
	tmp_norm=prod(tmp1,tmp_norm);
	matrix<double> m_norm=element_div(m,tmp_norm);
	matrix<double> cc=prod(trans(m_norm),m_norm);
	return cc;
}

// the struct is used for finding the maximum of the matrix, using iterator
struct find_max:public unary_function<double, void>{
	find_max() : maxx(-1) ,index(0),max_idx(0){}
	double maxx;
	int index;
	int max_idx;
	void operator()(double x) { 
		if(x>maxx){
			maxx=x;
			max_idx=index;
		}
		index++;}
};

// find the maximum of matrix, return the row and col num
boost::numeric::ublas::vector<int> Get_max(matrix<double> R){
    find_max result_m=for_each(R.data().begin(),R.data().end(),find_max());// iterator
	int idx_row=result_m.max_idx/R.size1();
	int idx_col=result_m.max_idx%R.size1();
	boost::numeric::ublas::vector<int> v_idx(2);
	v_idx(0)=idx_row;
	v_idx(1)=idx_col;
	return v_idx;
}
//the struct is used for initializing matrix. Get an all-0 matrix
struct zeromat{
	zeromat() {}
	double operator()() {return 0;}
} ;
